package com.gic.haoban.manage.service.service.out.impl;

import java.util.*;
import java.util.concurrent.TimeUnit;
import java.util.stream.Collector;
import java.util.stream.Collectors;

import com.alibaba.fastjson.JSONObject;
import com.gic.api.base.commons.Page;
import com.gic.commons.util.ToolUtil;
import com.gic.enterprise.api.dto.*;
import com.gic.enterprise.api.service.StoreService;
import com.gic.haoban.base.api.common.BasePageInfo;
import com.gic.haoban.base.api.common.ServiceResponse;
import com.gic.haoban.common.utils.PageUtil;
import com.gic.haoban.manage.api.dto.*;
import com.gic.haoban.manage.service.entity.*;
import com.gic.haoban.manage.service.service.StoreRangeService;
import com.gic.redis.data.util.RedisUtil;
import com.google.common.collect.Sets;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.gic.commons.util.EntityUtil;
import com.gic.enterprise.api.service.EnterpriseService;
import com.gic.enterprise.api.service.StoreGroupService;
import com.gic.haoban.common.utils.UuidUtil;
import com.gic.haoban.manage.api.service.WxEnterpriseRelatedApiService;
import com.gic.haoban.manage.service.dao.mapper.StaffMapper;
import com.gic.haoban.manage.service.dao.mapper.WxEnterpriseRelatedMapper;
import com.gic.haoban.manage.service.service.DepartmentService;
import com.gic.haoban.manage.service.service.StaffService;
import com.gic.haoban.manage.service.service.WxEnterpriseRelatedService;
import org.springframework.transaction.annotation.Transactional;

@Service
public class WxEnterpriseRelatedApiServiceImpl implements WxEnterpriseRelatedApiService {
    private static final Logger logger = LogManager.getLogger(WxEnterpriseRelatedApiServiceImpl.class);
    @Autowired
	private WxEnterpriseRelatedMapper wxEnterpriseRelatedMapper;
	@Autowired
	private WxEnterpriseRelatedService wxEnterpriseRelatedService;
	@Autowired
	private StoreGroupService storeGroupService;
	@Autowired
	private DepartmentService departmentService;
	@Autowired
	private StaffService staffService;
	@Autowired
	private StaffMapper staffMapper;

	@Autowired
	private EnterpriseService enterpriseService;

	@Autowired
	private StoreRangeService storeRangeService;

	@Autowired
	private StoreService storeService;

	private static final String FLUSH_HAOBAN_BIND_STORE = "flush_haoban_bind_store:";


	//获取门店详情
	public boolean isEnterpriseOver(String eid) {

		EnterpriseDTO enterprise = enterpriseService.getEnterpriseByEid(eid);
		if(enterprise == null || enterprise.getExpireTime() == null){
			return false;
		}
		if(enterprise.getExpireTime().getTime() > System.currentTimeMillis()){
			return false;
		}
		return true;

	}

	@Override
	public List<EnterpriseDetailDTO> listEnterpriseByWxEnterpriseId(
			String wxEnterpriseId) {
		
		List<TabHaobanWxEnterpriseRelated> relatedList = wxEnterpriseRelatedMapper.listByWxenterpriseId(wxEnterpriseId);
		List<EnterpriseDetailDTO> result = EntityUtil.changeEntityListByJSON(EnterpriseDetailDTO.class, relatedList);
		if (CollectionUtils.isEmpty(result)){
			return result;
		}
//		for (EnterpriseDetailDTO enterpriseDetailDTO : result) {
//			String enterpriseId = enterpriseDetailDTO.getEnterpriseId();
//			int gicAllCount = storeGroupService.gicDepartmentCount(enterpriseId);
//			int syncCount = departmentService.countSyncDepartmentByEnterpriseId(enterpriseId);
//			logger.info("【企业数量】gicAllCount={},syncCount={}",gicAllCount,syncCount);
//			int nodeCount = gicAllCount - syncCount;
//			enterpriseDetailDTO.setNodeCount(nodeCount);
//		}
		return result;
	}
	
//	public int getNodeCount(String enterpriseId,String parentId,int type) {
//		int nodeCount = 0;
//		List<GicTreeDTO> departmentList = storeGroupService.listGicTree(enterpriseId,type, parentId);
//		for (GicTreeDTO gicTreeDTO : departmentList) {
//			if(departmentService.getByRelatedId(gicTreeDTO.getId()) == null) {
//				nodeCount ++;
//			}
//			List<GicTreeDTO> departmentList2 = storeGroupService.listGicTree(enterpriseId,gicTreeDTO.getType(), gicTreeDTO.getId());
//			while(!departmentList2.isEmpty()) {
//				for (GicTreeDTO gicTreeDTO2 : departmentList) {
//					nodeCount += getNodeCount(enterpriseId,gicTreeDTO2.getId(),gicTreeDTO2.getType());
//				}
//			}
//		}
//		
//		return nodeCount;
//		
//	}

	@Override
	public String getGicEnterpriseIdByEnterpriseRelatedId(String wxEnterpriseRelatedId) {
		TabHaobanWxEnterpriseRelated tab = wxEnterpriseRelatedService.getByGicEnterpriseIdByEnterpriseRelatedId(wxEnterpriseRelatedId);
		if(tab != null) {
			return tab.getEnterpriseId();
		}
		
		return null;
	}

	@Override
	public boolean enterpriseUseNew(String enterpriseId) {
        if (null == enterpriseId) {
            return false;
        }
        List<String> eids = new ArrayList<>();
        eids.add(enterpriseId);
        List<TabHaobanWxEnterpriseRelated> retList = wxEnterpriseRelatedMapper.listEnterpriseByEnterpriseIds(eids);
        if (CollectionUtils.isEmpty(retList)) {
            return false;
		}
		return true;
	}

	@Override
	public EnterpriseDetailDTO getByEnterpriseId(String enterpriseId) {
		TabHaobanWxEnterpriseRelated tab = wxEnterpriseRelatedMapper.findOneByEnterpriseId(enterpriseId);
		EnterpriseDetailDTO detail = EntityUtil.changeEntityByJSON(EnterpriseDetailDTO.class, tab);
		return detail;
	}

	@Override
	public List<EnterpriseDetailDTO> queryAllHasGicEnterpriseId() {
		List<TabHaobanWxEnterpriseRelated> relatedList = wxEnterpriseRelatedMapper.queryAllHasGicEnterpriseId();
		List<EnterpriseDetailDTO> result = EntityUtil.changeEntityListByJSON(EnterpriseDetailDTO.class, relatedList);
		for (EnterpriseDetailDTO enterpriseDetailDTO : result) {
			String enterpriseId = enterpriseDetailDTO.getEnterpriseId();
			int gicAllCount = storeGroupService.gicDepartmentCount(enterpriseId);
			int syncCount = departmentService.countSyncDepartmentByEnterpriseId(enterpriseId);
			int nodeCount = gicAllCount - syncCount;
			enterpriseDetailDTO.setNodeCount(nodeCount);
		}
		return result;
	}

	@Override
	public List<EnterpriseDetailDTO> listEnterpriseByEnterpriseIds(
			List<String> enterpriseIds) {
		List<TabHaobanWxEnterpriseRelated> list= wxEnterpriseRelatedMapper.listEnterpriseByEnterpriseIds(enterpriseIds);
		List<EnterpriseDetailDTO> details = EntityUtil.changeEntityListByJSON(EnterpriseDetailDTO.class, list);
		return details;	
	}

	@Override
	public List<EnterpriseDetailDTO> queryBindGicEnterpriseByTime(String seqTime) {
		List<TabHaobanWxEnterpriseRelated> list= wxEnterpriseRelatedMapper.listEnterpriseByTime(seqTime);
		List<EnterpriseDetailDTO> details = EntityUtil.changeEntityListByJSON(EnterpriseDetailDTO.class, list);
		return details;
	}

	@Transactional(rollbackFor = Exception.class)
	@Override
	public ServiceResponse wxEnterpriseBind(WxEnterpriseRelationDetailDTO detailDTO) {
		ServiceResponse<Object> resp = new ServiceResponse<>();
		//校验是否分组被其它企业绑定
        if (!checkHasBindStoreGroup(detailDTO)) {
            logger.info("部分分组被其它企业绑定了：{}", JSONObject.toJSONString(detailDTO));
			resp.setCode(2);
			resp.setMessage("部分分组被其它企业绑定了！");
			return resp;
		}

		if (!checkRootEnterprise(detailDTO)) {
			logger.info("总部已经被设置不能存在多个总部：{}", JSONObject.toJSONString(detailDTO));
			resp.setCode(3);
			resp.setMessage("总部已经被设置不能存在多个总部！");
			return resp;
		}

		//校验关联已经企业绑定
		TabHaobanWxEnterpriseRelated tab = wxEnterpriseRelatedMapper.findOneByEIdAndWxEid(detailDTO.getEnterpriseId(), detailDTO.getWxEnterpriseId());
		if (tab != null && (!tab.getWxEnterpriseRelatedId().equals(detailDTO.getWxEnterpriseRelatedId()))) {
			logger.info("该企业已经被绑定过");
			resp.setCode(2);
			resp.setMessage("改企业已经被绑定！");
			return resp;
		}

		if (tab == null) {
			logger.info("该gicEnterpriseId已经绑定过了:{}", JSONObject.toJSONString(detailDTO));
			//保存企业关联关系
			tab = EntityUtil.changeEntityNew(TabHaobanWxEnterpriseRelated.class, detailDTO);
			tab.setStatusFlag(1);
			tab.setCreateTime(new Date());
			tab.setUpdateTime(new Date());
			tab.setWxEnterpriseRelatedId(UuidUtil.randomUUID());
			wxEnterpriseRelatedMapper.insertSelective(tab);
		} else {
			tab.setMemberOpenCardFlag(detailDTO.getMemberOpenCardFlag());
			tab.setWxEnterpriseType(detailDTO.getWxEnterpriseType());
			tab.setRangeType(1);
			tab.setUpdateTime(new Date());
			wxEnterpriseRelatedMapper.updateByPrimaryKeySelective(tab);
		}

		//保存门店范围
		storeRangeService.saveStoreRange(detailDTO.getRelations(), detailDTO.getEnterpriseId(), detailDTO.getWxEnterpriseId());
		//超级管理员新增
		TabHaobanStaff sStaff = staffMapper.selectSuperByWxEnterpriseId(detailDTO.getWxEnterpriseId());
		if (sStaff == null) {
			TabHaobanStaff staff = new TabHaobanStaff();
			staff.setStaffName("超级管理员");
			staff.setWxEnterpriseId(detailDTO.getWxEnterpriseId());
			staff.setSuperManagerFlag(1);
			staffService.add(staff);
		}

		return resp;
	}

	/**
	 * 如果是总部的话 校验是否是自己本身 并且可以执行下去
	 * 如果是代理 标识可以执行下  如果存在总部且不是保存的不能执行的
	 *
	 * @param detailDTO
	 * @return
	 */
	private boolean checkRootEnterprise(WxEnterpriseRelationDetailDTO detailDTO) {
		if (detailDTO.getWxEnterpriseType() == 1) {
			return true;
		}
		//校验总部
		TabHaobanWxEnterpriseRelated tab = wxEnterpriseRelatedMapper.findRootWxEnterpriseByEnterpriseId(detailDTO.getEnterpriseId());
		if (tab == null) {
			return true;
		}
		if (detailDTO.getWxEnterpriseRelatedId() != null && tab.getWxEnterpriseRelatedId().equals(detailDTO.getWxEnterpriseRelatedId())) {
			return true;
		}
		return false;

	}

	/**
	 * 校验是否包含被其它企业绑定
	 *
	 * @param detailDTO
	 * @return
	 */
	private boolean checkHasBindStoreGroup(WxEnterpriseRelationDetailDTO detailDTO) {
		//判断绑定情况
		List<StoreOrGroupInfoDTO> relations = detailDTO.getRelations();
		List<String> needBind = relations.stream().map(dto -> dto.getRelationId()).collect(Collectors.toList());
		List<TabStoreRange> storeRanges = storeRangeService.queryStoreRangeByRelationId(detailDTO.getEnterpriseId(), needBind);
		if (CollectionUtils.isEmpty(storeRanges)) {
			return true;
		}
		logger.info("绑定的部分：{}，{}", detailDTO.getWxEnterpriseId(), JSONObject.toJSONString(storeRanges));
		Set<TabStoreRange> mid = storeRanges.stream().filter(tab -> !tab.getWxEnterpriseId().equals(detailDTO.getWxEnterpriseId())).collect(Collectors.toSet());
		if (CollectionUtils.isEmpty(mid)) {
			return true;
		}
		return false;
	}

	@Override
	public ServiceResponse flushBindStoreByEnterpriseId(String enterpriseId, String wxEnterpriseId, String optStaffId, int channalCode) {
		ServiceResponse resp = new ServiceResponse();
		String key = FLUSH_HAOBAN_BIND_STORE + enterpriseId;
		Object cache = RedisUtil.getCache(key);
		if (null != cache) {
			logger.info("已经有人在操作刷新，不能同步操作：{},{}", enterpriseId, optStaffId);
			resp.setCode(2);
			resp.setMessage("有正在执行，不能同时操作");
			return resp;
		}
		RedisUtil.setCache(key, wxEnterpriseId, 120L, TimeUnit.SECONDS);

		logger.info("刷新绑定门店：eid:{},staffid:{}", enterpriseId, optStaffId);
		List<TabStoreRange> groupInfoDTOS = storeRangeService.queryAllBindRangeByEnterpriseId(enterpriseId);
		if (CollectionUtils.isEmpty(groupInfoDTOS)) {
			RedisUtil.delCache(key);
			logger.info("无需刷新：{}", enterpriseId);
			resp.setCode(3);
			resp.setMessage("没有要刷新的数据");
			return resp;
		}

		//根据微信企业分组 wxEnterpriseId->(List)groupId
		Map<String, Set<String>> relationMapByWxEid = groupInfoDTOS.stream().filter(tab -> tab.getRelationType() == 1)
				.collect(Collectors.groupingBy(tab -> tab.getWxEnterpriseId(), Collectors.mapping(tab -> tab.getRelationId(), Collectors.toSet())));

		//分组所在的企业map groupId->wxEnterpriseId
		Map<String, String> groupMap = groupInfoDTOS.stream().filter(tab -> tab.getRelationType() == 1)
				.collect(Collectors.toMap(tab -> tab.getRelationId(), tab -> tab.getWxEnterpriseId()));
        Set<String> rightGroupIds = groupMap.keySet();


		// 获取所有门店信息
		List<StoreSimpleDTO> storeDtos = storeService.listSimpleStore(enterpriseId, null, null);
		Map<String, List<String>> storeMapByGroupId = storeDtos.stream().collect(Collectors.groupingBy(dto -> dto.getStoreGroupId(), Collectors.mapping(dto -> dto.getStoreId(), Collectors.toList())));
        Map<String, StoreSimpleDTO> storeIdMap = storeDtos.stream().collect(Collectors.toMap(dto -> dto.getStoreId(), dto -> dto));

		//获取所有门店分组
		List<PowerStoreGroupDTO> storeGroupList = storeGroupService.getStoreGroupList(enterpriseId, null);
		Map<String, Set<String>> storeGroupChainMap = storeGroupList.stream().collect(Collectors.toMap(dto -> dto.getStoreGroupId(),
				dto -> Arrays.stream(dto.getStoreGroupChain().split("_")).filter(str -> StringUtils.isNotBlank(str)).collect(Collectors.toSet())));

		//获取集团门店的分组 然后根据分组来查询门店
		storeGroupChainMap.forEach((groupId, chainGroupIds) -> {
			Sets.SetView<String> midGroupIds = Sets.intersection(chainGroupIds, rightGroupIds);
			if (CollectionUtils.isEmpty(midGroupIds)) {
				return;
			}
			String midGroupId = null;

			//如果是1的话 加入对应企业
			if (midGroupIds.size() == 1) {
				midGroupId = midGroupIds.iterator().next();
			} else {
				//取交集层级最大 也就是冒泡最近的
				midGroupId = midGroupIds.stream().sorted(Comparator.comparing(mid -> storeGroupChainMap.get(mid).size()).reversed()).findFirst().get();
			}
			relationMapByWxEid.get(groupMap.get(midGroupId)).add(groupId);
		});

		//组装门店的id
		List<TabStoreRelation> ret = new ArrayList<>();
		relationMapByWxEid.forEach((wxEid, groupIds) -> {
            List<TabStoreRelation> midRelations = groupIds.stream().filter(mid -> storeMapByGroupId.containsKey(mid))
                    .flatMap(midGid -> storeMapByGroupId.get(midGid).stream())
                    .map(storeId -> {
                        StoreSimpleDTO simpleDTO = storeIdMap.get(storeId);
                        TabStoreRelation tabStoreRelation = new TabStoreRelation();
						tabStoreRelation.setWxEnterpriseId(wxEid);
                        tabStoreRelation.setStoreGroupId(simpleDTO.getStoreGroupId());
						tabStoreRelation.setStoreId(simpleDTO.getStoreId());
						tabStoreRelation.setStatusFlag(1);
                        tabStoreRelation.setStoreName(simpleDTO.getStoreName());
                        tabStoreRelation.setEnterpriseId(simpleDTO.getEnterpriseId());
                        tabStoreRelation.setStoreRelationId(ToolUtil.randomUUID());
						tabStoreRelation.setCreateTime(new Date());
						tabStoreRelation.setUpdateTime(new Date());
						return tabStoreRelation;
					}).collect(Collectors.toList());
			ret.addAll(midRelations);
		});

		storeRangeService.saveStoreRelations(ret, optStaffId, channalCode);
		RedisUtil.delCache(key);
		return resp;
	}

	@Override
	public WxEnterpriseRelationDetailDTO getEnterpriseBindInfo(String wxEnterpriseId, String enterpriseId) {
		TabHaobanWxEnterpriseRelated wxEnterpriseRelated = wxEnterpriseRelatedMapper.findOneByEIdAndWxEid(enterpriseId, wxEnterpriseId);
		if (null == wxEnterpriseRelated) {
			logger.info("企业未绑定：eid:{},weid:{}", enterpriseId, wxEnterpriseId);
			return null;
		}
		List<StoreOrGroupInfoDTO> infoDTOS = storeRangeService.queryBindStoreRange(enterpriseId, wxEnterpriseId);
		WxEnterpriseRelationDetailDTO detailDTO = new WxEnterpriseRelationDetailDTO();
		detailDTO.setRelations(infoDTOS);
		detailDTO.setEnterpriseId(enterpriseId);
		EnterpriseDTO enterpriseDTO = enterpriseService.getEnterpriseById(enterpriseId);
        if (null != enterpriseDTO) {
            detailDTO.setEnterpriseName(enterpriseDTO.getEnterpriseName());
		}
		detailDTO.setMemberOpenCardFlag(wxEnterpriseRelated.getMemberOpenCardFlag());
		detailDTO.setWxEnterpriseType(wxEnterpriseRelated.getWxEnterpriseType());
        detailDTO.setWxEnterpriseRelatedId(wxEnterpriseRelated.getWxEnterpriseRelatedId());
        return detailDTO;
	}

	@Override
	public List<WxEnterpriseDTO> listWxEnterpriseByEid(String enterpriseId) {
		List<TabHaobanWxEnterprise> wxEnterprises = wxEnterpriseRelatedService.listByEnterpriseId(enterpriseId);
		if (CollectionUtils.isEmpty(wxEnterprises)) {
			logger.info("没有关联企业");
			return null;
		}
		List<WxEnterpriseDTO> ret = EntityUtil.changeEntityListNew(WxEnterpriseDTO.class, wxEnterprises);
		return ret;
	}

	@Override
	public List<StoreRangeInfoDTO> listAllBindStoreOrGroup(String enterpriseId) {
		List<TabStoreRange> tabStoreRanges = storeRangeService.queryAllBindRangeByEnterpriseId(enterpriseId);
		return EntityUtil.changeEntityListNew(StoreRangeInfoDTO.class, tabStoreRanges);
	}

	@Override
	public Page<BindStoreInfoDTO> pageBindStoreByEnterpriseId(String wxEnterpriseId, String enterpriseId, String search, BasePageInfo pageInfo) {
		Page<TabStoreRelation> relationPage = storeRangeService.pageStoreRelation(enterpriseId, wxEnterpriseId, search, pageInfo);
		if (CollectionUtils.isEmpty(relationPage.getResult())) {
			return null;
		}
		Page<BindStoreInfoDTO> retPage = PageUtil.changeToCurrentPage(relationPage, BindStoreInfoDTO.class);
		List<String> storeIds = relationPage.getResult().stream().map(tab -> tab.getStoreId()).collect(Collectors.toList());
		List<StoreDTO> dtoList = storeService.queryStore(storeIds);
		retPage.setResult(EntityUtil.changeEntityListNew(BindStoreInfoDTO.class, dtoList));
		return retPage;
	}

	@Override
	public List<String> listStoreIdByWxEnterpriseId(String wxEnterpriseId) {
		List<String> list = storeRangeService.listStoreIdByWxEnterpriseId(wxEnterpriseId, null);
		return list;
	}
}
